/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#include <DataTypes/CommandData.hpp>
#include <Parser/MessageParser.hpp>

using namespace Dxc;
using namespace std;

CommandData::CommandData(Timestamp timestamp, const std::string& commandID, Value* value)
    : DxcData(timestamp), commands_()
{
    commands_.insert(Command(commandID, value));
}

CommandData::CommandData(Timestamp timestamp, const CommandSet& c)
    : DxcData(timestamp), commands_(c)
{
}

CommandData::CommandData(const CommandSet& c)
    : DxcData(), commands_(c)
{
}

ostream &CommandData::put(ostream & out) const
{
    out << "command @" << this->getTimeStamp();
    if (this->getSource() != "")
        out << " <" << this->getSource() << ">";

    out << " ";

    commands_.put(out);

    out << ";" << endl;

    return out;
}

const CommandSet* CommandData::getCommands() const
{
    return &commands_;
}

void CommandData::setCommands(const CommandSet& cmds)
{
    commands_.clear();
    commands_.insert(cmds.begin(), cmds.end());
}

std::string CommandData::getCommandID() const
{
    if (!commands_.empty())
        return commands_.begin()->getCommandID();
    else
        return "";
}

const Value* CommandData::getCommandValue() const
{
    if (!commands_.empty())
        return commands_.begin()->getValue();
    else
        return NULL;
}

CommandData* CommandData::clone() const
{
    return new CommandData(*this);
}

CommandData::CommandData()
    : DxcData(0)
{
}
