/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#include <DataTypes/DiagnosisData.hpp>

using namespace std;
using namespace Dxc;

DiagnosisData::DiagnosisData(Timestamp timestamp,
                             bool detectionSignal,
                             bool isolationSignal,
                             const CandidateSet& isolation,
                             const std::string& notes):
    DxcData(timestamp),
    detectionSignal_(detectionSignal),
    isolationSignal_(isolationSignal),
    candidateSet_(isolation),
    notes_(notes)
{
}

DiagnosisData::DiagnosisData(bool detectionSignal,
                             bool isolationSignal,
                             const CandidateSet& isolation,
                             const std::string& notes):
    DxcData(),
    detectionSignal_(detectionSignal),
    isolationSignal_(isolationSignal),
    candidateSet_(isolation),
    notes_(notes)
{
}

DiagnosisData::DiagnosisData(Timestamp timestamp,
                             bool detectionSignal,
                             bool isolationSignal,
                             const vector<FaultSet *>& candidates,
                             const vector<double>& weights,
                             const std::string& notes):
    DxcData(timestamp),
    detectionSignal_(detectionSignal),
    isolationSignal_(isolationSignal),
    candidateSet_(),
    notes_(notes)
{
    vector<FaultSet *>::const_iterator ci;
    vector<Literal *>::const_iterator fi;
    vector<double> tmpWeights(weights);

    for (ci = candidates.begin(); ci != candidates.end(); ci++)
    {
        Candidate c;
        c.setFaults(**ci);
        if (!weights.empty())
        {
            c.setWeight(tmpWeights.front());
            tmpWeights.erase(tmpWeights.begin());
        }

        candidateSet_.insert(c);
    }
}
DiagnosisData* DiagnosisData::clone() const
{
    return new DiagnosisData(*this);
}

ostream &DiagnosisData::put(ostream &out) const
{
    out << "diagnosis @" << this->getTimeStamp();
    if (this->getSource() != "")
        out << " <" << this->getSource() << ">";
    out << " detection = "
        << (this->getDetectionSignal() ? "true" : "false");
    out << ", isolation = "
        << (this->getIsolationSignal() ? "true" : "false");
    out << ", candidates = " << *(this->getCandidates());

    out << ", notes = '"
        << this->getNotes() << "';" << std::endl;

    out.flush();

    return out;
}

bool DiagnosisData::getDetectionSignal() const
{
    return detectionSignal_;
}

bool DiagnosisData::getIsolationSignal() const
{
    return isolationSignal_;
}

const CandidateSet* DiagnosisData::getCandidates() const
{
    return &candidateSet_;
}

string DiagnosisData::getNotes() const
{
    return notes_;
}
