/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#include <map>
#include <string>
#include <sstream>
#include <fstream>
#include <DataTypes/DxcException.hpp>
#include <DxcConfig.hpp>

#ifdef _WIN32
#pragma warning (disable:4996)
#endif
#include <cstdlib>

using namespace Dxc;
using namespace std;

DxcConfig *DxcConfig::ptr_ = NULL;
pthread_once_t DxcConfig::once_control_ = PTHREAD_ONCE_INIT;

void DxcConfig::init()
{
    ptr_ = new DxcConfig();
//    atexit(&destroy);
}

void DxcConfig::destroy()
{
    delete ptr_;
    ptr_ = NULL;
}

DxcConfig *DxcConfig::getInstance()
{
    (void) pthread_once(&once_control_, init);
    return ptr_;
}

DxcConfig::DxcConfig()
{
    char* dxcHome = getenv("DXC_HOME");
    if (!dxcHome)
    {
        throw DxcException("DXC_HOME is not set. Please set this environment variable.");
    }

    string dxcHomeStr(dxcHome);
    if (dxcHomeStr.at(dxcHomeStr.length() - 1) != DIR_DELIM)
        dxcHomeStr += DIR_DELIM;

    string cfgFile = dxcHomeStr + "Dxc.cfg";
    ifstream in(cfgFile.c_str());

    if (!in.is_open())
    {
        throw DxcException("Unable to open DXC Directory config file.");
    }

    unsigned int bufsz = 1024;
    string::size_type n, m;

    char *line = new char[bufsz];
    if (NULL == line)
    {
        throw DxcException("Error allocating memory.");
    }

    string strLine;

    do
    {
        in.getline(line, bufsz);
        strLine = string(line, in.gcount());

        while (in.rdstate() & in.failbit && in.gcount() > 0)
        {
            in.getline(line, bufsz);
            strLine += string(line, in.gcount());
        }

        n = strLine.find(' ');
        string key = strLine.substr(0, n);

        n = strLine.find_first_of('"');
        m = strLine.find_first_of('"', n + 1);
        string val = strLine.substr(n + 1, m - n - 1);

        myMap_.insert(make_pair(key, val));
    }
    while (in.gcount() > 0);

    delete[] line;
}

std::string DxcConfig::get(const std::string& key)
{
    std::map<std::string, std::string>::const_iterator i = myMap_.find(key);
    if (i != myMap_.end())
    {
        return myMap_.find(key)->second;
    }
    else
    {
        string ex("Config key ");
        ex += key + " not found.";
        throw DxcException(ex);
        return "";
    }
}

int DxcConfig::getInt(const std::string& key)
{
    std::map<std::string, std::string>::const_iterator i = myMap_.find(key);
    if (i != myMap_.end())
    {
        istringstream iss(myMap_.find(key)->second);

        int ret;
        iss >> ret;
        if (iss.fail())
        {
            string ex("Config key ");
            ex += key + " is badly formed or not an int.";
            throw DxcException(ex);
        }

        return ret;
    }
    else
    {
        string ex("Config key ");
        ex += key + " not found.";
        throw DxcException(ex);
    }
    return -1;
}

long DxcConfig::getLong(const std::string& key)
{
    std::map<std::string, std::string>::const_iterator i = myMap_.find(key);
    if (i != myMap_.end())
    {
        istringstream iss(myMap_.find(key)->second);

        long ret;
        iss >> ret;
        if (iss.fail())
        {
            string ex("Config key ");
            ex += key + " is badly formed or not an int.";
            throw DxcException(ex);
        }

        return ret;
    }
    else
    {
        string ex("Config key ");
        ex += key + " not found.";
        throw DxcException(ex);
    }
    return -1;
}

#ifdef _WIN32
#pragma warning (default:4996)
#endif
