/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.api.datatypes;

import java.io.OutputStream;
import java.io.PrintStream;

import org.dxc.api.datatypes.DxcData;

/**
 * Command datatype.
 */
public class CommandData extends DxcData {

    private CommandSet commands;

    /**
     * Constructor.
     * 
     * @param timestamp
     *            - message timestamp
     * @param commands
     *            - map of command id , value pairs
     */
    public CommandData(long timestamp, CommandSet commands) {
        super(timestamp);
        this.commands = (commands == null) ? new CommandSet() : commands;
    }

    /**
     * Constructor.
     * 
     * @param commands
     *            - map of command id, value pairs
     */
    public CommandData(CommandSet commands) {
        super();
        this.commands = (commands == null) ? new CommandSet() : commands;

    }

    /**
     * Empty constructor.
     */
    public CommandData() {
        super();
        this.commands = new CommandSet();

    }

    /**
     * Constructor.
     * 
     * @param timestamp
     *            - message timestamp
     * @param commandID
     *            - command ID string
     * @param value
     *            - the Value of the command
     */
    public CommandData(long timestamp, String commandID, Value value) {
        super(timestamp);
        this.commands = new CommandSet();
        commands.add(new Command((commandID == null) ? "" : commandID,
                (value == null) ? new StringValue("") : value));
    }

    /**
     * Constructor.
     * 
     * @param commandID
     *            - command ID string
     * @param value
     *            - the Value of the command
     */
    public CommandData(String commandID, Value value) {
        super();
        this.commands = new CommandSet();
        commands.add(new Command((commandID == null) ? "" : commandID,
                (value == null) ? new StringValue("") : value));
    }

    /**
     * Get first command ID string
     */
    public String getCommandID() {
        return this.commands.iterator().next().getCommandID();
    }

    /**
     * Get first command Value
     */
    public Value getValue() {
        return this.commands.iterator().next().getValue();
    }

    /**
     * Get commands
     */
    public CommandSet getCommands() {
        return this.commands;
    }

    /**
     * Set commands
     */
    public void setCommands(CommandSet commands) {
        this.commands = commands;
    }

    public OutputStream put(OutputStream os) {
        PrintStream out = new PrintStream(os);
        out.append("command @");
        out.append(Long.toString(this.getTimeStamp()) + " ");
        if (this.getSource() != null)
            out.append("<" + this.getSource() + "> ");

        out.append(commands.toString());

        out.append(";\n");
        return out;
    }

    public boolean equals(Object o) {
        if (o == this)
            return true;
        if ((o == null) || (o.getClass() != this.getClass()))
            return false;

        CommandData cd = (CommandData) o;

        if ((cd.getTimeStamp() != getTimeStamp())
                || (cd.getSource() != null && !cd.getSource().equals(
                        getSource()))
                || (!cd.getCommands().equals(getCommands())))
            return false;

        return true;
    }

    public int hashCode() {
        int hash = (int) getTimeStamp();
        hash = 31 * hash + getSource().hashCode();
        hash = 31 * hash + commands.hashCode();
        return hash;
    }
}
