package org.dxc.evaluator;

import java.io.File;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;

import org.w3c.dom.*;

import org.dxc.evaluator.metrics.*;

/*
 * Evaluator config parser.
 */
public class EvalConfig {
    private Map<String, MetricInfo> nameToMetric;
    private List<Double> rankToPoints;
    private boolean printStats;
    private static EvalConfig instance;

    public static EvalConfig getInstance() throws Exception {
        return instance != null ? instance : (instance = new EvalConfig(System
                .getenv("DXC_HOME")
                + "/evalCfg.xml"));
    }

    public Collection<MetricInfo> getMetrics() {
        return nameToMetric.values();
    }

    public MetricInfo getMetric(String name) {
        return nameToMetric.get(name);
    }

    public double getRankPoints(int rank) {
        try {
            return rankToPoints.get(rank);
        } catch (Exception ex) {
            return 0;
        }
    }

    public boolean printStats() {
        return printStats;
    }

    private EvalConfig(String filename) throws Exception {
        nameToMetric = new java.util.TreeMap<String, MetricInfo>();
        File f = new File(filename);

        DocumentBuilder db = DocumentBuilderFactory.newInstance()
                .newDocumentBuilder();
        Document doc = db.parse(f);
        doc.getDocumentElement().normalize();

        Node n = (Node) doc.getElementsByTagName("rankPoints").item(0);
        rankToPoints = parseDoubleArray(n.getFirstChild().getNodeValue());

        n = (Node) doc.getElementsByTagName("printStats").item(0);
        printStats = Boolean.parseBoolean(n.getFirstChild().getNodeValue());

        NodeList nl = doc.getElementsByTagName("metric");

        for (int i = 0; i < nl.getLength(); i++) {
            MetricInfo m = new MetricInfo((Element) nl.item(i));
            nameToMetric.put(m.getName(), m);
        }
    }

    private List<Double> parseDoubleArray(String s) {
        List<Double> il = new LinkedList<Double>();
        int beginIndex = 0, endIndex = 0;
        while (beginIndex != -1) {
            endIndex = s.indexOf(',', beginIndex);
            String ss = (endIndex < 0) ? s.substring(beginIndex) : s.substring(
                    beginIndex, endIndex);

            il.add(Double.parseDouble(ss));
            beginIndex = (endIndex < 0) ? -1 : endIndex + 1;
        }

        return il;
    }
}
