package org.dxc.evaluator.metrics;

import java.util.HashMap;
import java.util.Set;
import java.util.Map;

public class ConfusionMatrix {
    private double[][] matrix;
    private double count;
    private Map<String, Integer> idToIdx;
    private String[] idxToID;

    public static String NOMINAL = "Nominal";

    public ConfusionMatrix(Set<String> modes) {
        matrix = new double[modes.size()][modes.size()];
        idToIdx = new HashMap<String, Integer>();
        idxToID = new String[modes.size()];
        count = 0;

        int n = 0;

        for (String s : modes) {
            idToIdx.put(s, n);
            idxToID[n++] = s;
        }
    }

    public void addPrediction(String actual, String predicted, double weight) {
        int act = idToIdx.get(actual);
        int pre;
        try {
            pre = idToIdx.get(predicted);
        } catch (Exception ex) {
            pre = idToIdx.get(NOMINAL);
            System.out.println("Unknown fault mode ID in diagnosis: "
                    + predicted);
        }
        matrix[act][pre] += weight;
        count += weight;
    }

    public double percentCorrect() {
        double total = 0;
        for (int i = 0; i < matrix.length; i++) {
            total += matrix[i][i];
        }
        return total / (double) count;
    }

    public double totalGuesses() {
        double total = 0;
        for (int i = 0; i < matrix.length; i++)
            for (int j = 0; j < matrix.length; j++)
                total += matrix[i][j];
        return total;
    }

    public double totalCorrect() {
        double total = 0;
        for (int i = 0; i < matrix.length; i++) {
            total += matrix[i][i];
        }
        return total;
    }

    public double nominalCorrect() {
        int i = idToIdx.get(NOMINAL);
        return matrix[i][i];
    }

    public void print() {
        System.out.println("Indices:");
        for (String s : idxToID) {
            System.out.print(s + " , ");
        }
        System.out.println();
        for (int i = 0; i < matrix.length; i++) {
            for (int j = 0; j < matrix.length; j++)
                System.out.print(matrix[i][j] + ", ");
            System.out.println();
        }

    }
}
